FusionHUD = LibStub("AceAddon-3.0"):NewAddon("FusionHUD", "AceConsole-3.0");

local FusionHUDMasterFrame = CreateFrame("Frame", nil, UIParent);
local _G = _G;
local FusionHUDBars = {};
local FusionHUDBarPrototype = {};
local FusionHUDBarSettingsPrototype = {};
local FusionHUDSettings = { 
	mainOptions = {
		testToggle = true
	},
	barOptions = {
	}
};

FusionHUDOptions = {
    name = "Fusion HUD",
    handler = FusionHUD,
    type = 'group',
    args = {
		mainOptions = {
			name = "Main Menu",
			order = 1,
			type = "group",
			desc = "Main menu for FusionHUD",
			args = {
				testToggle = {
					name = "Test Toggle",
					type = "toggle",
					desc = "Toggle Checkbox for testing",
					get = 	function() return FusionHUDSettings.mainOptions.testToggle end,
					set = 	function() 
								FusionHUDSettings.mainOptions.testToggle = not FusionHUDSettings.mainOptions.testToggle;
								if FusionHUDSettings.mainOptions.testToggle then
									FusionHUD:Print("I'm On!");
								else
									FusionHUD:Print("I'm Off!");
								end
							end,
				},
				newBar = {
					name = "New Bar",
					type = "execute",
					desc = "Creates a new HUD bar",
					func = 	function()
								FusionHUD:Print(#FusionHUDBars + 1);
								FusionHUDBars[#FusionHUDBars + 1] = FusionHUDBarPrototype:New(#FusionHUDBars + 1);
							end,
				},	
			}		
		},
		bars = {
			name = "Bar List",
			order = 2,
			type = "group",
			desc = "Listing of each HUD bar",
			args = {
--[[				newBar = {
					name = "New Bar",
					type = "execute",
					desc = "Creates a new HUD bar",
					func = 	function()
								FusionHUD:Print(#FusionHUDBars + 1);
								FusionHUDBars[#FusionHUDBars + 1] = FusionHUDBarPrototype:New(#FusionHUDBars + 1);
							end,
				},	--]]
			}
		},
    },
};

function FusionHUDBarPrototype:New(index)		--New Bar object, initializes the frames and textures
	self.barIndex = index;
	self.parentFrame = "FusionHUDMasterFrame";
	self.draggable = false;
	self.myPoint = "CENTER";
	self.parentPoint = "CENTER";
	self.orientation = "LEFT";
	self.textureName = "Interface\\AddOns\\FusionHUD\\Graphics\\White Straight Bar Base";
	self.containerTextureName = "Interface\\AddOns\\FusionHUD\\Graphics\\Straight Bar Glass Container";
	self.height = 400;
	self.width = 15;
	self.barType = "Health";
	self.barTarget = "player";
	self.totals = true;
	self.text = "EEEEEEE";
	self.warningColor = false;
	--Set up the bar's frame
	self.frame = CreateFrame("Frame", nil, FusionHUDMasterFrame);
	self.frame:ClearAllPoints();
	self.frame:SetPoint(self.myPoint, FusionHUDMasterFrame, self.parentPoint);
	self.frame:SetHeight(self.height);
	self.frame:SetWidth(self.width);
	--Set up the bar texture
	self.frame.barTexture = self.frame:CreateTexture(nil, "BACKGROUND");
	self.frame.barTexture:SetTexture(self.textureName, false);
	self.frame.barTexture:ClearAllPoints();
	self.frame.barTexture:SetPoint("BOTTOM", self.frame, "BOTTOM");
	self.frame.barTexture:SetHeight(self.height);
	self.frame.barTexture:SetWidth(self.width);
	--Set up the bar background/scale texture
	self.frame.containerTexture = self.frame:CreateTexture(nil, "OVERLAY");
	self.frame.containerTexture:SetTexture(self.containerTextureName, false);
	self.frame.containerTexture:ClearAllPoints();
	self.frame.containerTexture:SetPoint("BOTTOM", self.frame, "BOTTOM");
	self.frame.containerTexture:SetHeight(self.height);
	self.frame.containerTexture:SetWidth(self.width);
	self.frame.containerTexture:SetTexCoord(0,1,1,0);
	self.frame.containerTexture:SetBlendMode("BLEND");
	--Set up bar text
	self.frame.text = self.frame:CreateFontString(nil, "LOW");
	self.frame.text:ClearAllPoints();
	self.frame.text:SetPoint("TOP", self.frame, "BOTTOM");
	self.frame.text:SetFont("Fonts\\FRIZQT__.TTF", 12, "OUTLINE, MONOCHROME");
	self.frame.text:SetJustifyH("CENTER");
	self.frame.text:SetJustifyV("TOP");
	self.frame.text:SetHeight(30);
	self.frame.text:SetWidth(150);
	FusionHUDOptions.args.bars.args["bar" .. self.barIndex] = {};
	FusionHUD:Print(index);

	FusionHUDOptions.args.bars.args["bar" .. self.barIndex].name = "Bar " .. self.barIndex;
	FusionHUDOptions.args.bars.args["bar" .. self.barIndex].type = "group";
	FusionHUDOptions.args.bars.args["bar" .. self.barIndex].args = {
		lock = {
			name = "Locked",
			type = "toggle",
			desc = "Unlocks the bar for repositioning",
			get = 	function() return not FusionHUDBars[index].draggable end;
			set = 	function() 
						FusionHUDBars[self.barIndex].draggable = not FusionHUDBars[self.barIndex].draggable;
--						FusionHUDBars[barIndex]:LockManager()
					end;
		},
		barType = {
			name = "Bar Type",
			type = "select",
			desc = "The bar can track either Health or Power(Mana/Runic Power/Energy/Focus)",
			style = "dropdown",
			values = { Power = "Power", Health = "Health" },
			get = 	function() return FusionHUDBars[self.barIndex].barType end;
			set = 	function(key, value)
						FusionHUD:Print(key, value);
						FusionHUDBars[self.barIndex].barType = value;
					end
		}
	}
	self.Update = 	function()	-- The bar update function
						if self.barType == "Health" then  --check for type of bar, power or health
							--set bar height and text for a health bar
							self.frame.barTexture:SetHeight(400*UnitHealth(self.barTarget)/UnitHealthMax(self.barTarget));
							self.frame.barTexture:SetTexCoord(0,1,UnitHealth(self.barTarget)/UnitHealthMax(self.barTarget),0);
							self.frame.barTexture:SetVertexColor(0,1,0,1);
							self.frame.text:SetVertexColor(0,1,0,1);
							self.frame.text:SetText(UnitHealth(self.barTarget) .. "/" .. UnitHealthMax(self.barTarget));
						elseif self.barType == "Power" then
							local powerType, powerTypeString = UnitPowerType(self.barTarget);
							--first color the bar the appropriate color
							if powerTypeString == "RAGE" then
								self.frame.barTexture:SetVertexColor(0.8,0,0,1);
								self.frame.text:SetVertexColor(0.8,0,0,1);
							elseif powerTypeString == "RUNIC_POWER" then
								self.frame.barTexture:SetVertexColor(0.6,0,0,1);
								self.frame.text:SetVertexColor(0.6,0,0,1);
							elseif powerTypeString == "MANA" then
								self.frame.barTexture:SetVertexColor(0,0,0.8,1);
								self.frame.text:SetVertexColor(0,0,0.8,1);
							elseif powerTypeString == "FOCUS" then
								self.frame.barTexture:SetVertexColor(0.8,0.3,0,1);
								self.frame.text:SetVertexColor(0.8,0.3,0,1);
							else
								self.frame.barTexture:SetVertexColor(1,1,0,1);
								self.frame.text:SetVertexColor(1,1,0,1);
							end
							--set bar height and text
							if (400*UnitMana(self.barTarget)/UnitManaMax(self.barTarget)) == 0 then
								self.frame.barTexture:SetHeight(1);
							else
								self.frame.barTexture:SetHeight(400*UnitMana(self.barTarget)/UnitManaMax(self.barTarget));
							end
							self.frame.barTexture:SetTexCoord(0,1,UnitMana("player")/UnitManaMax("player"),0);
							self.frame.text:SetText(UnitMana(self.barTarget) .. "/" .. UnitManaMax(self.barTarget));
						end
					end
	return self;
end
--[[
function FusionHUDBarSettingsPrototype:New(barIndex)
	self.name = "Bar " .. barIndex;
	self.type = "group";
	self.args = {
		lock =	{
			name = "Locked",
			type = "toggle",
			desc = "Unlocks the bar for repositioning",
			get = 	function() return not FusionHUDBars[barIndex] end;
			set = 	function() 
						FusionHUDBars[barIndex] = not FusionHUDBars[barIndex];
--						FusionHUDBars[barIndex]:LockManager()
					end;
		}
	}
	return self;
end
--]]
function FusionHUD:OnInitialize()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("FusionHUD", FusionHUDOptions);
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("FusionHUD", "FusionHUD");
	self.RegisterChatCommand("fh", "ChatCommand");
	self.RegisterChatCommand("fusionhud", "ChatCommand");
	
	FusionHUDMasterFrame:SetClampedToScreen(true);
	FusionHUDMasterFrame:SetPoint("CENTER",UIParent,"CENTER");
	FusionHUDMasterFrame:SetFrameStrata("LOW");
	FusionHUDMasterFrame:SetHeight(400);
	FusionHUDMasterFrame:SetWidth(400);
	FusionHUDMasterFrame:Show();
	FusionHUDMasterFrame:SetAlpha(1);
	
	FusionHUDMasterFrame:SetScript("OnUpdate", 	function() FusionHUD:OnUpdate()	end); 
	
	--test frame --
	FusionHUDBars[1] = FusionHUDBarPrototype:New(1);
	FusionHUDBars[1].frame:SetPoint("RIGHT", FusionHUDMasterFrame, "LEFT");
--	FusionHUDBars[1].barType = "power";
	--test frame --
	
end

function FusionHUD:ChatCommand(input)
	if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(WelcomeHome, "fh", "FusionHUD", input)
    end
end	

function FusionHUD:OnUpdate()
	local i = 1;
	while FusionHUDBars[i]  do
		FusionHUDBars[i].Update();
		i = i + 1;
	end
end